@@ -239,6 +239,7 @@ pub static FUNC_RENAME: Lazy<HashMap<&str, &str>> = Lazy::new(|| hashmap! {
 	"cv_ocl_Kernel_set_int_const_UMatR" => "+_umat",
 	"cv_ocl_ProgramSource_ProgramSource_const_StringR" => "from_str",
 	"cv_ocl_Program_getPrefix_const_StringR" => "+_build_flags",
+	"cv_ogl_Buffer_create_Size_int_Target_bool" => "+_size",
 	"cv_operatorA_const_MatExprR_const_MatExprR" => "+_matexpr_matexpr",
 	"cv_operatorA_const_MatExprR_const_MatR" => "+_matexpr_mat",
 	"cv_operatorA_const_MatExprR_const_ScalarR" => "+_matexpr_scalar",
@@ -332,8 +333,8 @@ pub static FUNC_RENAME: Lazy<HashMap<&str, &str>> = Lazy::new(|| hashmap! {
 	"cv_vconcat_const_MatX_size_t_const__OutputArrayR" => "-", // duplicate of cv_vconcat_VectorOfMat_Mat, but with pointers
 
 	// ### cudaimgproc ###
-	"cv_cuda_histEven_const__InputArrayR_GpuMat_X__4__int_X__4__int_X__4__int_X__4__StreamR" => "-", // slice of boxed objects
-	"cv_cuda_histRange_const__InputArrayR_GpuMat_X__4__const_GpuMat_X__4__StreamR" => "-", // slice of boxed objects
+	"cv_cuda_histEven_const__InputArrayR_GpuMatXX_intXX_intXX_intXX_StreamR" => "-", // slice of boxed objects
+	"cv_cuda_histRange_const__InputArrayR_GpuMatXX_const_GpuMatXX_StreamR" => "-", // slice of boxed objects
 
 	// ### dnn ###
 	"cv_dnn_DictValue_DictValue_bool" => "from_bool",
@@ -471,6 +472,8 @@ pub static FUNC_RENAME: Lazy<HashMap<&str, &str>> = Lazy::new(|| hashmap! {
 	"cv_groupRectangles_vector_Rect_R_vector_int_R_int_double" => "+_weights",
 	"cv_groupRectangles_vector_Rect_R_int_double_vector_int_X_vector_double_X" => "+_levelweights",
 
+	"cv_QRCodeDetector_detectAndDecodeMulti_const_const__InputArrayR_vector_string_R_const__OutputArrayR_const__OutputArrayR" => "-", // fixme: stores data to Vector<String>, that doesn't work yet
+
 	// ### optflow ###
 	"cv_optflow_GPCTrainingSamples_operator_cv_optflow_GPCSamplesVector" => "-", // support of "operator &" missing
 
@@ -679,63 +682,43 @@ pub static ELEMENT_EXPORT_TWEAK: Lazy<HashMap<&str, fn (&mut ExportConfig)>> = L
 	"cv::dnn::KeypointsModel" => ExportConfig::make_boxed as _, // marked as simple from OpenCV 4.5.2
 	"cv::dnn::Model" => ExportConfig::make_boxed as _,
 	"cv::dnn::SegmentationModel" => ExportConfig::make_boxed as _, // marked as simple from OpenCV 4.5.2
-	"cv::dnn::TextDetectionModel_DB" => ExportConfig::make_boxed as _, // incorrectly marked as simple
-	"cv::dnn::TextDetectionModel_EAST" => ExportConfig::make_boxed as _, // incorrectly marked as simple
-	"cv::dnn::TextRecognitionModel" => ExportConfig::make_boxed as _, // incorrectly marked as simple
+	"cv::dnn::TextDetectionModel" => ExportConfig::make_boxed as _, // inappropriately marked as simple
+	"cv::dnn::TextDetectionModel_DB" => ExportConfig::make_boxed as _, // inappropriately marked as simple
+	"cv::dnn::TextDetectionModel_EAST" => ExportConfig::make_boxed as _, // inappropriately marked as simple
+	"cv::dnn::TextRecognitionModel" => ExportConfig::make_boxed as _, // inappropriately marked as simple
 });
 
 /// set of functions that should have unsafe in their declaration, element is Func.identifier()
-pub static FUNC_UNSAFE: Lazy<HashSet<&str>> = Lazy::new(|| hashset! {
+pub static FUNC_UNSAFE: Lazy<HashSet<FuncId>> = Lazy::new(|| hashset! {
 	// allocates uninitialized memory
-	"cv_Mat_Mat_Size_int",
-	"cv_Mat_Mat_const_vector_int_R_int",
-	"cv_Mat_Mat_int_const_intX_int",
-	"cv_Mat_Mat_int_int_int",
-	"cv_Mat_create_Size_int",
-	"cv_Mat_create_const_vector_int_R_int",
-	"cv_Mat_create_int_const_intX_int",
-	"cv_Mat_create_int_int_int",
-	"cv_UMat_UMat_Size_int_UMatUsageFlags",
-	"cv_UMat_UMat_int_const_intX_int_UMatUsageFlags",
-	"cv_UMat_UMat_int_int_int_UMatUsageFlags",
-	"cv_UMat_create_Size_int_UMatUsageFlags",
-	"cv_UMat_create_const_vector_int_R_int_UMatUsageFlags",
-	"cv_UMat_create_int_const_intX_int_UMatUsageFlags",
-	"cv_UMat_create_int_int_int_UMatUsageFlags",
-	"cv__OutputArray_createSameSize_const_const__InputArrayR_int",
-	"cv__OutputArray_create_const_Size_int_int_bool_int",
-	"cv__OutputArray_create_const_int_const_intX_int_int_bool_int",
-	"cv__OutputArray_create_const_int_int_int_int_bool_int",
-	// allows passing arbitrary data
-	"cv_Mat_Mat_Size_int_voidX_size_t",
-	"cv_Mat_Mat_const_vector_int_R_int_voidX_const_size_tX",
-	"cv_Mat_Mat_int_const_intX_int_voidX_const_size_tX",
-	"cv_Mat_Mat_int_int_int_voidX_size_t",
-	"cv_Mat_setPropData_unsigned_charX",
-	"cv_UMatData_setPropData_unsigned_charX",
-	"cv_cuda_GpuMat_GpuMat_int_int_int_voidX_size_t",
-	"cv_cuda_GpuMat_GpuMat_Size_int_voidX_size_t",
-	// no bounds checking
-	"cv_Mat_ptr_const_const_intX",
-	"cv_Mat_ptr_const_int",
-	"cv_Mat_ptr_const_intX",
-	"cv_Mat_ptr_const_int_int",
-	"cv_Mat_ptr_const_int_int_int",
-	"cv_Mat_ptr_int",
-	"cv_Mat_ptr_int_int",
-	"cv_Mat_ptr_int_int_int",
+	FuncId::new("cv::Mat::Mat", ["size", "type"]),
+	FuncId::new("cv::Mat::Mat", ["sizes", "type"]),
+	FuncId::new("cv::Mat::Mat", ["ndims", "sizes", "type"]),
+	FuncId::new("cv::Mat::Mat", ["rows", "cols", "type"]),
+	FuncId::new("cv::Mat::create", ["size", "type"]),
+	FuncId::new("cv::Mat::create", ["sizes", "type"]),
+	FuncId::new("cv::Mat::create", ["ndims", "sizes", "type"]),
+	FuncId::new("cv::Mat::create", ["rows", "cols", "type"]),
+	FuncId::new("cv::UMat::UMat", ["size", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::UMat", ["ndims", "sizes", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::UMat", ["rows", "cols", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::create", ["size", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::create", ["size", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::create", ["ndims", "sizes", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::create", ["sizes", "type", "usageFlags"]),
+	FuncId::new("cv::UMat::create", ["rows", "cols", "type", "usageFlags"]),
+	FuncId::new("cv::_OutputArray::createSameSize", ["arr", "mtype"]),
 	// pointer to internal data
-	"cv_dnn_Dict_ptr_const_StringR",
-	"cv_dnn_Dict_ptr_const_const_StringR",
+	FuncId::new("cv::dnn::Dict::ptr", ["key"]),
 	// takes reference and stores it for the lifetime of an object (fixme: add lifetime management)
-	"cv_cuda_GpuMat_GpuMat_AllocatorX",
-	"cv_cuda_GpuMat_GpuMat_Size_int_AllocatorX",
-	"cv_cuda_GpuMat_GpuMat_Size_int_Scalar_AllocatorX",
-	"cv_cuda_GpuMat_GpuMat_const__InputArrayR_AllocatorX",
-	"cv_cuda_GpuMat_GpuMat_int_int_int_AllocatorX",
-	"cv_cuda_GpuMat_GpuMat_int_int_int_Scalar_AllocatorX",
-	"cv_cuda_GpuMat_setPropAllocator_AllocatorX",
-	"cv_cuda_GpuMat_setDefaultAllocator_AllocatorX", // fixme, should take 'static
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["allocator"]),
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["size", "type", "allocator"]),
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["size", "type", "s", "allocator"]),
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["arr", "allocator"]),
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["rows", "cols", "type", "allocator"]),
+	FuncId::new("cv::cuda::GpuMat::GpuMat", ["rows", "cols", "type", "s", "allocator"]),
+	FuncId::new("cv::cuda::GpuMat::allocator", ["val"]),
+	FuncId::new("cv::cuda::GpuMat::setDefaultAllocator", ["allocator"]), // fixme, should take 'static
 });
 
 pub static IMPLEMENTED_FUNCTION_LIKE_MACROS: Lazy<HashSet<&str>> = Lazy::new(|| hashset! {
@@ -810,26 +793,52 @@ pub static FORCE_CONSTANT_METHOD: Lazy<HashSet<&str>> = Lazy::new(|| hashset! {
 	"cv::UMat::step",
 });
 
-/// (cpp_fullname, argument count)
-pub static FORCE_NOEXCEPT: Lazy<HashSet<(&str, usize)>> = Lazy::new(|| hashset! {
+pub static FORCE_INFALLIBLE: Lazy<HashSet<FuncId>> = Lazy::new(|| hashset! {
+	// just returns static/constant data
+	FuncId::new("cv::noArray", []),
+	FuncId::new("cv::getVersionMajor", []),
+	FuncId::new("cv::getVersionMinor", []),
+	FuncId::new("cv::getVersionRevision", []),
+	// not doing anything that can cause an exception
+	FuncId::new("cv::Mat::empty", []),
+	FuncId::new("cv::Mat::total", []),
+	FuncId::new("cv::Mat::isContinuous", []),
+	FuncId::new("cv::Mat::isSubmatrix", []),
+	FuncId::new("cv::Mat::elemSize1", []),
+	FuncId::new("cv::Mat::type", []),
+	FuncId::new("cv::Mat::depth", []),
+	FuncId::new("cv::Mat::channels", []),
+	FuncId::new("cv::UMat::empty", []),
+	FuncId::new("cv::UMat::total", []),
+	FuncId::new("cv::UMat::isContinuous", []),
+	FuncId::new("cv::UMat::isSubmatrix", []),
+	FuncId::new("cv::UMat::elemSize1", []),
+	FuncId::new("cv::UMat::type", []),
+	FuncId::new("cv::UMat::depth", []),
+	FuncId::new("cv::UMat::channels", []),
+	FuncId::new("cv::SparseMat::elemSize", []),
+	FuncId::new("cv::SparseMat::elemSize1", []),
+	FuncId::new("cv::SparseMat::type", []),
+	FuncId::new("cv::SparseMat::depth", []),
+	FuncId::new("cv::SparseMat::channels", []),
 	// marked CV_NOEXCEPT since OpenCV 4.5.2, propagate those changes to earlier versions
-	("cv::Mat::Mat", 0),
-	("cv::MatSize::MatSize", 1),
-	("cv::MatSize::dims", 0),
-	("cv::MatSize::operator const int*", 0),
-	("cv::MatStep::MatStep", 0),
-	("cv::MatStep::operator[]", 1),
-	("cv::UMat::UMat", 1),
-	("cv::ocl::Context::Context", 0),
-	("cv::ocl::Device::Device", 0),
-	("cv::ocl::Image2D::Image2D", 0),
-	("cv::ocl::Kernel::Kernel", 0),
-	("cv::ocl::KernelArg::KernelArg", 0),
-	("cv::ocl::Platform::Platform", 0),
-	("cv::ocl::PlatformInfo::PlatformInfo", 0),
-	("cv::ocl::Program::Program", 0),
-	("cv::ocl::ProgramSource::ProgramSource", 0),
-	("cv::ocl::Queue::Queue", 0),
+	FuncId::new("cv::Mat::Mat", []),
+	FuncId::new("cv::MatSize::MatSize", ["_p"]),
+	FuncId::new("cv::MatSize::dims", []),
+	FuncId::new("cv::MatSize::operator const int *", []),
+	FuncId::new("cv::MatStep::MatStep", []),
+	FuncId::new("cv::MatStep::operator[]", ["i"]),
+	FuncId::new("cv::UMat::UMat", ["usageFlags"]),
+	FuncId::new("cv::ocl::Context::Context", []),
+	FuncId::new("cv::ocl::Device::Device", []),
+	FuncId::new("cv::ocl::Image2D::Image2D", []),
+	FuncId::new("cv::ocl::Kernel::Kernel", []),
+	FuncId::new("cv::ocl::KernelArg::KernelArg", []),
+	FuncId::new("cv::ocl::Platform::Platform", []),
+	FuncId::new("cv::ocl::PlatformInfo::PlatformInfo", []),
+	FuncId::new("cv::ocl::Program::Program", []),
+	FuncId::new("cv::ocl::ProgramSource::ProgramSource", []),
+	FuncId::new("cv::ocl::Queue::Queue", []),
 });
 
 /// cpp_fullname => ( rust_fullname, cpp_fullname )
@@ -892,95 +901,111 @@ pub static IMPLEMENTED_CONST_GENERICS: Lazy<HashSet<&str>> = Lazy::new(|| hashse
 });
 
 #[derive(Clone, Copy, Debug)]
-pub enum ArgumentOverride {
+pub enum ArgOverride {
 	Nullable,
-	Slice,
 	NullableSlice,
+	Slice,
 	LenForSlice(&'static str, usize),
+	StringAsBytes,
 }
 
-pub static ARGUMENT_OVERRIDE: Lazy<HashMap<FuncId, HashMap<&str, ArgumentOverride>>> = Lazy::new(|| hashmap! {
+pub static ARGUMENT_OVERRIDE: Lazy<HashMap<FuncId, HashMap<&str, ArgOverride>>> = Lazy::new(|| hashmap! {
 	FuncId::new("cv::Mat::at", ["idx"]) => hashmap! {
-		"idx" => ArgumentOverride::Slice
+		"idx" => ArgOverride::Slice
 	},
 	FuncId::new("cv::Mat::ptr", ["idx"])  => hashmap! {
-		"idx" => ArgumentOverride::Slice
+		"idx" => ArgOverride::Slice
 	},
 	FuncId::new("cv::Mat::Mat", ["sizes", "type", "data", "steps"]) => hashmap! {
-		"steps" => ArgumentOverride::NullableSlice,
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"steps" => ArgOverride::NullableSlice,
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::Mat::Mat", ["ndims", "sizes", "type", "s"]) => hashmap! {
-		"steps" => ArgumentOverride::NullableSlice,
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"steps" => ArgOverride::NullableSlice,
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::Mat::Mat", ["ndims", "sizes", "type", "data", "steps"]) => hashmap! {
-		"steps" => ArgumentOverride::NullableSlice,
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"steps" => ArgOverride::NullableSlice,
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::Mat::zeros", ["ndims", "sz", "type"]) => hashmap! {
-		"sz" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sz", 1),
+		"sz" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sz", 1),
 	},
 	FuncId::new("cv::Mat::ones", ["ndims", "sz", "type"]) => hashmap! {
-		"sz" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sz", 1),
+		"sz" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sz", 1),
 	},
 	FuncId::new("cv::Mat::create", ["ndims", "sizes", "type"]) => hashmap! {
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::Mat::reshape", ["cn", "newndims", "newsz"]) => hashmap! {
-		"newsz" => ArgumentOverride::Slice,
-		"newndims" => ArgumentOverride::LenForSlice("newsz", 1),
+		"newsz" => ArgOverride::Slice,
+		"newndims" => ArgOverride::LenForSlice("newsz", 1),
 	},
 	FuncId::new("cv::SparseMat::Hdr::Hdr", ["_dims", "_sizes", "_type"]) => hashmap! {
-		"_sizes" => ArgumentOverride::Slice,
-		"_dims" => ArgumentOverride::LenForSlice("_sizes", 1),
+		"_sizes" => ArgOverride::Slice,
+		"_dims" => ArgOverride::LenForSlice("_sizes", 1),
 	},
 	FuncId::new("cv::UMat::UMat", ["ndims", "sizes", "type", "usageFlags"]) => hashmap! {
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::UMat::UMat", ["ndims", "sizes", "type", "s", "usageFlags"]) => hashmap! {
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::UMat::create", ["ndims", "sizes", "type", "usageFlags"]) => hashmap! {
-		"sizes" => ArgumentOverride::Slice,
-		"ndims" => ArgumentOverride::LenForSlice("sizes", 1),
+		"sizes" => ArgOverride::Slice,
+		"ndims" => ArgOverride::LenForSlice("sizes", 1),
 	},
 	FuncId::new("cv::_OutputArray::create", ["dims", "size", "type", "i", "allowTransposed", "fixedDepthMask"]) => hashmap! {
-		"size" => ArgumentOverride::Slice,
-		"dims" => ArgumentOverride::LenForSlice("size", 1),
+		"size" => ArgOverride::Slice,
+		"dims" => ArgOverride::LenForSlice("size", 1),
 	},
 	FuncId::new("cv::mixChannels", ["src", "dst", "fromTo", "npairs"]) => hashmap! {
-		"fromTo" => ArgumentOverride::Slice,
-		"npairs" => ArgumentOverride::LenForSlice("from_to", 2),
+		"fromTo" => ArgOverride::Slice,
+		"npairs" => ArgOverride::LenForSlice("from_to", 2),
 	},
 	FuncId::new("cv::createTrackbar", ["trackbarname", "winname", "value", "count", "onChange", "userdata"]) => hashmap! {
-		"value" => ArgumentOverride::Nullable,
+		"value" => ArgOverride::Nullable,
 	},
 	FuncId::new("cv::minMaxLoc", ["src", "minVal", "maxVal", "minLoc", "maxLoc", "mask"]) => hashmap! {
-		"minVal" => ArgumentOverride::Nullable,
-		"maxVal" => ArgumentOverride::Nullable,
-		"minLoc" => ArgumentOverride::Nullable,
-		"maxLoc" => ArgumentOverride::Nullable,
+		"minVal" => ArgOverride::Nullable,
+		"maxVal" => ArgOverride::Nullable,
+		"minLoc" => ArgOverride::Nullable,
+		"maxLoc" => ArgOverride::Nullable,
 	},
 	FuncId::new("cv::minMaxLoc", ["a", "minVal", "maxVal", "minIdx", "maxIdx"]) => hashmap! {
-		"minVal" => ArgumentOverride::Nullable,
-		"maxVal" => ArgumentOverride::Nullable,
-		"minIdx" => ArgumentOverride::Nullable,
-		"maxIdx" => ArgumentOverride::Nullable,
+		"minVal" => ArgOverride::Nullable,
+		"maxVal" => ArgOverride::Nullable,
+		"minIdx" => ArgOverride::Nullable,
+		"maxIdx" => ArgOverride::Nullable,
 	},
 	FuncId::new("cv::minMaxIdx", ["src", "minVal", "maxVal", "minIdx", "maxIdx", "mask"]) => hashmap! {
-		"minVal" => ArgumentOverride::Nullable,
-		"maxVal" => ArgumentOverride::Nullable,
-		"minIdx" => ArgumentOverride::Nullable,
-		"maxIdx" => ArgumentOverride::Nullable,
+		"minVal" => ArgOverride::Nullable,
+		"maxVal" => ArgOverride::Nullable,
+		"minIdx" => ArgOverride::Nullable,
+		"maxIdx" => ArgOverride::Nullable,
+	},
+	FuncId::new("cv::decodeQRCode", ["in", "points", "decoded_info", "straight_qrcode"]) => hashmap! {
+		"decoded_info" => ArgOverride::StringAsBytes,
+	},
+	FuncId::new("cv::QRCodeDetector::decode", ["img", "points", "straight_qrcode"]) => hashmap! {
+		"return" => ArgOverride::StringAsBytes,
+	},
+	FuncId::new("cv::QRCodeDetector::decodeCurved", ["img", "points", "straight_qrcode"]) => hashmap! {
+		"return" => ArgOverride::StringAsBytes,
+	},
+	FuncId::new("cv::QRCodeDetector::detectAndDecode", ["img", "points", "straight_qrcode"]) => hashmap! {
+		"return" => ArgOverride::StringAsBytes,
+	},
+	FuncId::new("cv::QRCodeDetector::detectAndDecodeCurved", ["img", "points", "straight_qrcode"]) => hashmap! {
+		"return" => ArgOverride::StringAsBytes,
 	},
 });
 
@@ -1112,6 +1137,7 @@ pub static GENERATOR_MODULE_TWEAKS: Lazy<HashMap<&str, ModuleTweak>> = Lazy::new
 		generate_types: vec![
 			// for findContours()
 			"std::vector<cv::Vec4i>",
+			"std::vector<std::vector<cv::Point>>",
 			// for HoughLines()
 			"std::vector<cv::Vec2f>",
 			"std::vector<cv::Vec2d>",
