@@ -4,7 +4,8 @@
 {{attributes}}
 {{visibility}}{{unsafety_decl}}fn {{name}}{{generic_decl}}({{decl_args}}){{rv_rust_full}} {
 	{{pre_call_args}}
-	{{prefix}}{{unsafety_call}}{ sys::{{identifier}}({{call_args}}) }.into_result(){{ret_map}}{{suffix}}
+	let ret = {{unsafety_call}}{ sys::{{identifier}}({{call_args}}) }{{ret_convert}};
+	{{ret_map}}
 	{{post_call_args}}
 }
 
