@@ -10,13 +10,6 @@ impl core::ToInputArray for {{rust_localalias}} {
 	}
 }
 
-impl core::ToInputArray for &{{rust_localalias}} {
-	#[inline]
-	fn input_array(&self) -> Result<core::_InputArray> {
-		(*self).input_array()
-	}
-}
-
 impl core::ToOutputArray for {{rust_localalias}} {
 	#[inline]
 	fn output_array(&mut self) -> Result<core::_OutputArray> {
@@ -27,13 +20,6 @@ impl core::ToOutputArray for {{rust_localalias}} {
 	}
 }
 
-impl core::ToOutputArray for &mut {{rust_localalias}} {
-	#[inline]
-	fn output_array(&mut self) -> Result<core::_OutputArray> {
-		(*self).output_array()
-	}
-}
-
 impl core::ToInputOutputArray for {{rust_localalias}} {
 	#[inline]
 	fn input_output_array(&mut self) -> Result<core::_InputOutputArray> {
@@ -44,9 +30,4 @@ impl core::ToInputOutputArray for {{rust_localalias}} {
 	}
 }
 
-impl core::ToInputOutputArray for &mut {{rust_localalias}} {
-	#[inline]
-	fn input_output_array(&mut self) -> Result<core::_InputOutputArray> {
-		(*self).input_output_array()
-	}
-}
+input_output_array_ref_forward! { {{rust_localalias}} }
